from portal.service.errors import IncorrectPasswordError, UnauthorizedError
from portal.service.service import find_user, find_component


def login_user(gitlab_access_token=None, password=None, username=None):
    if gitlab_access_token:
        return auth_gitlab_access_token(
            username=username,
            gitlab_access_token=gitlab_access_token
        )

    return auth_username_password(username=username, password=password)


def auth_gitlab_access_token(username, gitlab_access_token):
    """Authentication using gitlab token

    Steps:
        Send token to gitlab
        Verify that token is for user

    Args:
        gitlab_access_token:

    Returns: the authenticated user

    """


def auth_username_password(username, password):
    user = find_user(username)

    if user.verify_password(password=password):
        return user

    raise IncorrectPasswordError()


def login_component(name: str, ip_address: str, secret: str):
    client = find_component(name)
    if client.secret != secret:
        raise UnauthorizedError(note="Invalid component secret.")
    if ip_address != client.ip_address:
        raise UnauthorizedError(note="Different source ip address for component login request.")
    return client
