from portal.database.models import User, Course, Component, Role


def check_component(component: Component, permissions: list, course: Course=None) -> bool:
    """Checks component permissions for a course

    Args:
        component(Component): Component instance
        course(Course): Course instance
        permissions(list): List of permissions required

    Returns(bool): True if checks has passed

    """
    # TODO
    if not isinstance(component, Component):
        return False
    return True


def check_user(user: User, course: Course, permissions: list) -> bool:
    """Checks a user's permissions based on his roles in the course

    Args:
        user(User): User instance
        course(Course): Course instance
        permissions(list): Permissions list

    Returns(bool): True if checks pass

    """
    if not isinstance(user, User):
        return False
    if check_sysadmin(user):
        return True
    roles = user.get_roles_in_course(course=course)

    return all(__evaluate_permission_for_roles(roles, perm) for perm in permissions)


def check_sysadmin(user: User) -> bool:
    """Checks whether the user is sysadmin

    Args:
        user(User): User instance

    Returns(bool): True if checks pass
    """
    if not isinstance(user, User):
        return False
    return user.is_admin


def __evaluate_permission_for_role(role: Role, permission: str, default=False) -> bool:
    """Evaluates permission for a role

    If the permission doesn't exist, False is returned.
    Args:
        role(Role): the role to check
        permission(str): Permission name
        default(bool): Default behaviour when the permission does not exist

    Returns(bool): True if permission is True in the role

    """
    return getattr(role, permission, default=default)


def __evaluate_permission_for_roles(roles: list, permission: str) -> bool:
    """Evaluates permissions for all a list of roles
    Args:
        roles(list): Role list
        permission(str): Permission name

    Returns(bool): True if any of the roles allows the permission

    """
    return any(__evaluate_permission_for_role(role, permission) for role in roles)
